from .field import HashidField, BigHashidField, HashidAutoField, BigHashidAutoField
from .hashid import Hashid

__title__ = 'Django Hashid Field'
__version__ = "3.2.0"
__author__ = 'Nathan Shafer'
__license__ = 'MIT License'
__copyright__ = 'Copyright 2017-2021 Nathan Shafer'

# Version synonym
VERSION = __version__
